{-# LANGUAGE DataKinds #-}
{-# LANGUAGE OverloadedStrings #-}

module Jon.Main
    ( main
    , runFunction
    , runQuery
    , runIns
    ) where

import Control.Exception (bracket)
import Database.Beam
import Database.Beam.Postgres
import Servant
import Servant.Swagger
import Servant.Swagger.UI
import System.Environment
import Network.Wai.Handler.Warp (run)

import qualified Data.ByteString.Char8 as BS8

import Jon.Server (JonAPI, jonSwaggerDoc, server)

main :: IO ()
main = withGarfieldConn $ \conn ->
    run 8080 $ serve p $ server conn :<|> swaggerSchemaUIServer jonSwaggerDoc
    where
        p :: Proxy (JonAPI :<|> SwaggerSchemaUI "swagger" "swagger.json")
        p = Proxy

withGarfieldConn = bracket
    (do pass <- getEnv "JON_PASS"
        connectPostgreSQL $ BS8.pack $ "host=localhost dbname=garfield password=" ++ pass)
    close

runQuery
    :: (FromBackendRow Postgres (QExprToIdentity e), Projectible Postgres e)
    => Q Postgres db QBaseScope e
    -> IO [QExprToIdentity e]
runQuery q = withGarfieldConn $ \conn -> runBeamPostgresDebug putStrLn conn $ runSelectReturningList $ select q

runIns
    :: SqlInsert Postgres table
    -> IO ()
runIns i = withGarfieldConn $ \conn -> runBeamPostgresDebug putStrLn conn $ runInsert i

runFunction
    :: (Connection -> IO a)
    -> IO a
runFunction = withGarfieldConn
